<?php

class ImageHandler {

    // 固定值前缀，用于生成新的文件名
    private $prefix = '';

    // 缓存图片目录
    private $cacheImagesDirectory = '_files/cache_images';

    public function __construct() {
        // 构造函数，可以添加初始化逻辑
    }

    /**
     * 获取指定目录及其子目录中的所有有效图片，并进行重命名操作。
     *
     * @param string $directory 目标目录
     */
    public function getAllImages($directory) {
        try {
            // 验证目录的有效性
            $this->validateDirectory($directory);

            // 获取指定目录及其子目录中的所有有效图片
            $allImages = $this->getImages($directory);

            if (!empty($allImages)) {
                // 重命名所有图片
                $this->renameImages($directory, $allImages);
            } else {
                // 未找到有效图片的提示
                echo '在指定目录及其子目录中未找到有效的图片。请检查输入的目录是否正确，并确保目录及其子目录中包含有效的图片文件。';
            }
        } catch (Exception $e) {
            // 捕获并输出异常信息
            echo $e->getMessage();
        }
    }

    /**
     * 删除缓存图片目录及其下所有文件。
     */
    public function deleteCacheImages() {
        try {
            // 验证缓存目录的有效性
            $this->validateDirectory($this->cacheImagesDirectory);

            // 递归删除缓存目录及其下所有文件
            $this->recursiveRemoveDirectory($this->cacheImagesDirectory);

            // 输出成功删除的信息
            echo '已删除缓存目录: ' . $this->cacheImagesDirectory . '<br>';
        } catch (Exception $e) {
            // 捕获并输出异常信息
            echo $e->getMessage();
        }
    }

    /**
     * 重命名指定目录及其子目录中的所有图片。
     *
     * @param string $directory 目标目录
     * @param array $images 包含所有图片相对路径的数组
     */
    private function renameImages($directory, $images) {
        foreach ($images as $image) {
            $filePath = $directory . DIRECTORY_SEPARATOR . $image;
            $newName = $this->generateNewName($image);
            $newPath = $directory . DIRECTORY_SEPARATOR . $newName;

            // 重命名图片
            $result = rename($filePath, $newPath);

            if ($result) {
                // 输出已重命名的图片信息
                echo '图片已重命名为: ' . $newPath . '<br>';
            }
        }

        // 递归处理子目录
        $subdirectories = $this->getSubdirectories($directory);
        foreach ($subdirectories as $subdirectory) {
            $this->renameImages($subdirectory, $this->getImages($subdirectory));
        }
    }

    /**
     * 生成新的文件名。
     *
     * @param string $oldName 原文件名
     * @return string 新文件名
     */
    private function generateNewName($oldName) {
        $pathInfo = pathinfo($oldName);
        $extension = $pathInfo['extension'];

        // 生成一个混合的24位随机数和字母
        $randomPart = bin2hex(random_bytes(12)); // 12 bytes => 24 characters

        // 新的文件名
        return $this->prefix . $randomPart . '.' . $extension;
    }

    /**
     * 验证目录的有效性。
     *
     * @param string $directory 待验证的目录
     * @throws Exception 如果目录为空或无效，则抛出异常
     */
    private function validateDirectory($directory) {
        if (empty($directory)) {
            throw new Exception('目录不能为空。');
        }

        $directory = realpath($directory);

        if ($directory === false || !is_dir($directory)) {
            throw new Exception('无效的目录。');
        }
    }

    /**
     * 获取目录中的所有图片。
     *
     * @param string $directory 目标目录
     * @return array 包含所有图片相对路径的数组
     */
    private function getImages($directory) {
        $images = [];
        $files = scandir($directory);

        foreach ($files as $file) {
            $filePath = $directory . DIRECTORY_SEPARATOR . $file;

            if (is_dir($filePath) && !in_array($file, ['.', '..'])) {
                $imagesFromSubdirectory = $this->getImages($filePath);
                $images = array_merge($images, $imagesFromSubdirectory);
            } elseif (is_file($filePath)) {
                $validExtensions = ['jpg', 'png', 'gif', 'jpeg', 'mp4'];
                $extension = strtolower(pathinfo($file, PATHINFO_EXTENSION));

                if (in_array($extension, $validExtensions)) {
                    // 获取图片相对路径
                    $relativePath = $this->getRelativePath($filePath, $directory);
                    $images[] = $relativePath;
                }
            }
        }

        return $images;
    }

    /**
     * 获取相对路径。
     *
     * @param string $absolutePath 绝对路径
     * @param string $rootDirectory 根目录
     * @return string 相对路径
     */
    private function getRelativePath($absolutePath, $rootDirectory) {
        return str_replace($rootDirectory, '', $absolutePath);
    }

    /**
     * 获取目录的子目录。
     *
     * @param string $directory 目标目录
     * @return array 包含所有子目录的数组
     */
    private function getSubdirectories($directory) {
        $subdirectories = [];
        $files = scandir($directory);

        foreach ($files as $file) {
            $filePath = $directory . DIRECTORY_SEPARATOR . $file;

            if (is_dir($filePath) && !in_array($file, ['.', '..'])) {
                $subdirectories[] = $filePath;
            }
        }

        return $subdirectories;
    }

    /**
     * 递归删除目录及其下所有文件。
     *
     * @param string $directory 目标目录
     */
    private function recursiveRemoveDirectory($directory) {
        foreach (glob("{$directory}/*") as $file) {
            if (is_dir($file)) {
                $this->recursiveRemoveDirectory($file);
            } else {
                unlink($file);
                // 输出已删除的缓存图片信息
                echo '已删除缓存图片: ' . $file . '<br>';
            }
        }
        rmdir($directory);
    }
}

// 创建 ImageHandler 实例
$imageHandler = new ImageHandler();

// 调用 getAllImages 方法，传入目录参数
$imageHandler->getAllImages($_GET['type']);

// 调用 deleteCacheImages 方法
$imageHandler->deleteCacheImages();
?>
